@@ -149,10 +149,12 @@ class _Project(_Base):
         with open(path, 'rb') as f:
             self._make_request('POST', url, files={'modelfile': f})
 
-    def pull_model(self, filepath, datasource_key=None, round=None, throw_error=False):
+    def pull_model(self, filepath, datasource_key=None, round=None, federated_model=None, throw_error=False):
         params = {}
         if round is not None:
             params['round'] = round
+        if federated_model is not None:
+            params['federatedModel'] = federated_model
 
         if datasource_key is None:
             url = f'{self._get_route()}/projects/{self.key}/models'
@@ -194,21 +196,7 @@ class _Datasource(_Base):
         if model_path is not None:
             self.trainers[key]['model_path'] = model_path
 
-        for p in self.dfl.project_participants:
-            project_key = p['project_key']
-            datasource_key = p['datasource_key']
-            threads_key = get_threads_key(project_key, datasource_key)
-            '''
-            The first time attach_datasource() we might find 1 previous project for that ds1.
-            So project_participants = [item1]
-            The next time it is called it might also find 1 previous project for ds2
-            So project_participants = [item1, item2]
-            We only want create a thread for items that we haven't done so already
-            '''
-            if threads_key in self.dfl.on_round_threads:
-                continue
-            info = self._make_request('GET', f'/projects/{project_key}')
-            self.dfl.on_round_threads[threads_key] = gevent.spawn(self.dfl.train_and_test_callback, datasource_key, info)
+        self.dfl.initiate_project_participants(should_spawn_threads=True)
 
 
 class DynamoFL(_Base):
@@ -240,7 +228,7 @@ class DynamoFL(_Base):
         j = json.loads(res)
         if j['event'] == 'client-info':
             self.instance_id = j['data']['id']
-            # self.initiate_project_participants(should_fetch_bridges=True, should_spawn_threads=True, reset=True)
+            self.initiate_project_participants(should_fetch_bridges=True, should_spawn_threads=True)
 
         if j['event'] == 'new-project':
             project_key = j['data']['projectKey']
@@ -344,7 +332,7 @@ class DynamoFL(_Base):
         if not yes_submission or not yes_stats:
             # Pull
             print(f'>>> ({project_key}-{datasource_key}) Waiting to download round ({prev_round}) federated model...')
-            project.pull_model(federated_model_path, round=prev_round)
+            project.pull_model(federated_model_path, round=prev_round, datasource_key=datasource_key, federated_model=True)
 
         # Test
         if not yes_stats:
@@ -373,16 +361,15 @@ class DynamoFL(_Base):
             print('Done.')
             print()
 
-    def initiate_project_participants(self, should_fetch_bridges=False, should_spawn_threads=False, ds=None, reset=False):
-        if reset:
-            self.project_participants = []
+    def initiate_project_participants(self, should_fetch_bridges=False, should_spawn_threads=False, ds=None):
 
         if should_fetch_bridges:
             if ds:
-                datasources = [ds]
+                datasources = [ds] # targeting specific datasource, we form an array with only one ds key
             else:
-                datasources = self.datasources
-                
+                datasources = self.datasources # all datasources as a dict
+                self.project_participants = [] 
+
             for ds_key in datasources:
                 j = self._make_request('GET', '/bridges', params={'datasourceKey': ds_key})
                 for i in j['data']:
@@ -430,18 +417,11 @@ class DynamoFL(_Base):
         else:
             self._make_request('POST', '/datasources', params=params)
 
-        # get current projects of newly attached datasource
-        j = self._make_request('GET', '/bridges', params={'datasourceKey': key})
-        for i in j['data']:
-            self.project_participants.append({
-                'project_key': i['projectKey'],
-                'datasource_key': i['datasourceKey'],
-                'trainer_key': i['trainerKey'],
-                'hyper_param_values': i['hyperParamValues']
-            })
 
         ds = _Datasource(self, key)
         self.datasources[key] = ds
+        self.initiate_project_participants(should_fetch_bridges=True, ds=key)
+
         return ds
 
     def delete_datasource(self, key):
